@@ -6,8 +6,10 @@ Configuration file parser and other default configuration variables.
 TODO: change settings into a class.
 """
 import configparser
+import glob
 import json
 import os
+import re
 import shlex
 
 from epoptes.common.constants import G_INSTANCE
@@ -97,7 +99,67 @@ def read_shell_file(filename):
         return {}
 
 
-def read_groups(filename):
+def read_groups_ltsp(filename):
+    """Parse group information from a global /etc/ltsp/ltsp.conf file.
+    """
+    data = read_plain_file(filename)
+    if not data:
+        return [], []
+    # Dict of dicts, e.g. groups["group"]["mac"] = "hostname"
+    groups = {}
+    group = None
+    mac = None
+    hostname = None
+    for line in data:
+        # line is already stripped, no need to remove surrounding space
+        s = re.search(r"^#\s*EPOPTES_GROUP\s*=\s*(.*)$", line)
+        if s:
+            group = s.groups()[0]
+            # HOSTNAMEs are ignored after EPOPTES_GROUP=None
+            if group == "None":
+                group = None
+            mac = None
+            continue
+        s = re.search(r"^\[([0-9a-fA-F:]{17})\]", line)
+        if s:
+            if group:
+                mac = s.groups()[0]
+            continue
+        s = re.search(r"^\[(.*)\]", line)
+        if s:
+            # Invalidating mac, found no_mac_section
+            mac = None
+            continue
+        s = re.search(r"^\s*HOSTNAME\s*=\s*(.*)$", line)
+        if not s or not group or not mac:
+            continue
+        # We have a hostname under a matching group and mac; process it
+        hostname = s.groups()[0]
+        if group not in groups:
+            groups[group] = {}
+        groups[group][mac] = hostname
+        # Invalidate mac in order to process only the first HOSTNAME under it
+        mac = None
+
+    clients_struct = {}
+    for group in groups:
+        for mac in groups[group]:
+            alias = groups[group][mac]
+            new = structs.Client('offline', mac, '', alias)
+            clients_struct[group + mac] = new
+
+    groups_struct = []
+    for group in groups:
+        members = {}
+        for mac in groups[group]:
+            members[clients_struct[group + mac]] = {}
+
+        groups_struct.append(structs.Group(group, members))
+
+    return clients_struct.values(), groups_struct
+
+
+def read_groups_json(filename):
     """Parse a JSON file and create the appropriate group and client objects.
     Return a 2-tuple with a client instances list and a group instances list.
     """
@@ -167,8 +229,12 @@ def save_groups(filename, model):
         LOG.e(exc)
 
 
+system = {}
 # The system settings are shared with epoptes-client, that's why the caps.
-system = read_shell_file('/etc/default/epoptes')
+for fname in glob.glob('/etc/default/epoptes') + \
+        glob.glob('/etc/epoptes/common/*.conf') + \
+        glob.glob('/etc/epoptes/server/*.conf'):
+    system.update(read_shell_file(fname))
 # TODO: check if the types, e.g. PORT=int, may cause problems.
 system.setdefault('PORT', 789)
 system.setdefault('SOCKET_GROUP', 'epoptes')
